@@ -22,35 +22,35 @@ along with pktools.  If not, see <http://www.gnu.org/licenses/>.
 #include <iostream>
 #include <cmath>
 #include "Filter2d.h"
-#include "Histogram.h"
+#include "StatFactory.h"
 // #include "imageclasses/ImgUtils.h"
 
-Filter2d::Filter2d::Filter2d(void)
+filter2d::Filter2d::Filter2d(void)
   : m_noValue(0)
 {
 }
 
-Filter2d::Filter2d::Filter2d(const Vector2d<double> &taps)
+filter2d::Filter2d::Filter2d(const Vector2d<double> &taps)
   : m_taps(taps), m_noValue(0)
 {
 }
 
-void Filter2d::Filter2d::setTaps(const Vector2d<double> &taps)
+void filter2d::Filter2d::setTaps(const Vector2d<double> &taps)
 {
   m_taps=taps;
 }
 
-void Filter2d::Filter2d::smoothNoData(const ImgReaderGdal& input, ImgWriterGdal& output, int dim)
+void filter2d::Filter2d::smoothNoData(const ImgReaderGdal& input, ImgWriterGdal& output, int dim)
 {
   smoothNoData(input, output,dim,dim);
 }
 
-void Filter2d::Filter2d::smooth(const ImgReaderGdal& input, ImgWriterGdal& output, int dim)
+void filter2d::Filter2d::smooth(const ImgReaderGdal& input, ImgWriterGdal& output, int dim)
 {
   smooth(input, output,dim,dim);
 }
 
-void Filter2d::Filter2d::smoothNoData(const ImgReaderGdal& input, ImgWriterGdal& output, int dimX, int dimY)
+void filter2d::Filter2d::smoothNoData(const ImgReaderGdal& input, ImgWriterGdal& output, int dimX, int dimY)
 {
   m_taps.resize(dimY);
   for(int j=0;j<dimY;++j){
@@ -61,7 +61,7 @@ void Filter2d::Filter2d::smoothNoData(const ImgReaderGdal& input, ImgWriterGdal&
   filter(input,output,false,true,true);
 }
 
-void Filter2d::Filter2d::smooth(const ImgReaderGdal& input, ImgWriterGdal& output, int dimX, int dimY)
+void filter2d::Filter2d::smooth(const ImgReaderGdal& input, ImgWriterGdal& output, int dimX, int dimY)
 {
   m_taps.resize(dimY);
   for(int j=0;j<dimY;++j){
@@ -73,7 +73,7 @@ void Filter2d::Filter2d::smooth(const ImgReaderGdal& input, ImgWriterGdal& outpu
 }
 
     
-void Filter2d::Filter2d::filter(const ImgReaderGdal& input, ImgWriterGdal& output, bool absolute, bool normalize, bool noData)
+void filter2d::Filter2d::filter(const ImgReaderGdal& input, ImgWriterGdal& output, bool absolute, bool normalize, bool noData)
 {
   int dimX=m_taps[0].size();//horizontal!!!
   int dimY=m_taps.size();//vertical!!!
@@ -182,7 +182,7 @@ void Filter2d::Filter2d::filter(const ImgReaderGdal& input, ImgWriterGdal& outpu
 }
 
 
-void Filter2d::Filter2d::majorVoting(const string& inputFilename, const string& outputFilename,int dim,const vector<int> &prior)
+void filter2d::Filter2d::majorVoting(const string& inputFilename, const string& outputFilename,int dim,const vector<int> &prior)
 {
   bool usePriors=true;  
   if(prior.empty()){
@@ -383,34 +383,39 @@ void Filter2d::Filter2d::majorVoting(const string& inputFilename, const string&
 //   output.close();
 // }
 
-void Filter2d::Filter2d::median(const string& inputFilename, const string& outputFilename,int dim, bool disc)
+void filter2d::Filter2d::median(const string& inputFilename, const string& outputFilename,int dim, bool disc)
 {
   ImgReaderGdal input;
   ImgWriterGdal output;
   input.open(inputFilename);
   output.open(outputFilename,input);
-  doit(input,output,MEDIAN,dim,disc);
+  doit(input,output,"median",dim,disc);
 }
 
-void Filter2d::Filter2d::var(const string& inputFilename, const string& outputFilename,int dim, bool disc)
+void filter2d::Filter2d::var(const string& inputFilename, const string& outputFilename,int dim, bool disc)
 {
   ImgReaderGdal input;
   ImgWriterGdal output;
   input.open(inputFilename);
   output.open(outputFilename,input);
-  doit(input,output,VAR,dim,disc);
+  doit(input,output,"var",dim,disc);
 }
 
-void Filter2d::Filter2d::doit(const ImgReaderGdal& input, ImgWriterGdal& output, int method, int dim, short down, bool disc)
+void filter2d::Filter2d::doit(const ImgReaderGdal& input, ImgWriterGdal& output, const std::string& method, int dim, short down, bool disc)
 {
   doit(input,output,method,dim,dim,down,disc);
 }
 
-void Filter2d::Filter2d::doit(const ImgReaderGdal& input, ImgWriterGdal& output, int method, int dimX, int dimY, short down, bool disc)
+void filter2d::Filter2d::doit(const ImgReaderGdal& input, ImgWriterGdal& output, const std::string& method, int dimX, int dimY, short down, bool disc)
 {
   assert(dimX);
   assert(dimY);
-  Histogram hist;
+  statfactory::StatFactory stat;
+  const char* pszMessage;
+  void* pProgressArg=NULL;
+  GDALProgressFunc pfnProgress=GDALTermProgress;
+  double progress=0;
+  pfnProgress(progress,pszMessage,pProgressArg);
   for(int iband=0;iband<input.nrOfBand();++iband){
     Vector2d<double> inBuffer(dimY,input.nrOfCol());
     vector<double> outBuffer((input.nrOfCol()+down-1)/down);
@@ -426,11 +431,6 @@ void Filter2d::Filter2d::doit(const ImgReaderGdal& input, ImgWriterGdal& output,
       }
       ++indexJ;
     }
-    const char* pszMessage;
-    void* pProgressArg=NULL;
-    GDALProgressFunc pfnProgress=GDALTermProgress;
-    double progress=0;
-    pfnProgress(progress,pszMessage,pProgressArg);
     for(int y=0;y<input.nrOfRow();++y){
       if(y){//inBuffer already initialized for y=0
 	//erase first line from inBuffer
@@ -488,7 +488,6 @@ void Filter2d::Filter2d::doit(const ImgReaderGdal& input, ImgWriterGdal& output,
 	    }
 	    if(!masked){
               vector<short>::const_iterator vit=m_class.begin();
-              //todo: test if this works (only add occurrence if within defined classes)!
               if(!m_class.size())
                 ++occurrence[inBuffer[indexJ][indexI]];
               else{
@@ -501,64 +500,100 @@ void Filter2d::Filter2d::doit(const ImgReaderGdal& input, ImgWriterGdal& output,
             }
 	  }
         }
-        switch(method){
-        case(MEDIAN):
-          outBuffer[x/down]=hist.median(windowBuffer);
+        switch(getFilterType(method)){
+        case(filter2d::median):
+          if(windowBuffer.empty())
+            outBuffer[x/down]=m_noValue;
+          else
+            outBuffer[x/down]=stat.median(windowBuffer);
           break;
-        case(VAR):{
-          outBuffer[x/down]=hist.var(windowBuffer);
+        case(filter2d::var):{
+          if(windowBuffer.empty())
+            outBuffer[x/down]=m_noValue;
+          else
+            outBuffer[x/down]=stat.var(windowBuffer);
           break;
         }
-        case(MEAN):{
-          outBuffer[x/down]=hist.mean(windowBuffer);
+        case(filter2d::stdev):{
+          if(windowBuffer.empty())
+            outBuffer[x/down]=m_noValue;
+          else
+            outBuffer[x/down]=sqrt(stat.var(windowBuffer));
           break;
         }
-        case(MIN):{
-          outBuffer[x/down]=hist.min(windowBuffer);
+        case(filter2d::mean):{
+          if(windowBuffer.empty())
+            outBuffer[x/down]=m_noValue;
+          else
+            outBuffer[x/down]=stat.mean(windowBuffer);
           break;
         }
-        case(ISMIN):{
-          outBuffer[x/down]=(hist.min(windowBuffer)==windowBuffer[dimX*dimY/2])? 1:0;
+        case(filter2d::min):{
+          if(windowBuffer.empty())
+            outBuffer[x/down]=m_noValue;
+          else
+           outBuffer[x/down]=stat.min(windowBuffer);
           break;
         }
-        case(MINMAX):{
+        case(filter2d::ismin):{
+           if(windowBuffer.empty())
+            outBuffer[x/down]=m_noValue;
+          else
+            outBuffer[x/down]=(stat.min(windowBuffer)==windowBuffer[dimX*dimY/2])? 1:0;
+          break;
+        }
+        case(filter2d::minmax):{
           double min=0;
           double max=0;
-          hist.minmax(windowBuffer,windowBuffer.begin(),windowBuffer.end(),min,max);
-          if(min!=max)
-            outBuffer[x/down]=0;
+          if(windowBuffer.empty())
+            outBuffer[x/down]=m_noValue;
+          else{
+            stat.minmax(windowBuffer,windowBuffer.begin(),windowBuffer.end(),min,max);
+            if(min!=max)
+              outBuffer[x/down]=0;
+            else
+              outBuffer[x/down]=windowBuffer[dimX*dimY/2];//centre pixels
+          }
+          break;
+        }
+        case(filter2d::max):{
+          if(windowBuffer.empty())
+            outBuffer[x/down]=m_noValue;
           else
-            outBuffer[x/down]=windowBuffer[dimX*dimY/2];//centre pixels
+            outBuffer[x/down]=stat.max(windowBuffer);
           break;
         }
-        case(MAX):{
-          outBuffer[x/down]=hist.max(windowBuffer);
+        case(filter2d::ismax):{
+          if(windowBuffer.empty())
+            outBuffer[x/down]=m_noValue;
+          else
+            outBuffer[x/down]=(stat.max(windowBuffer)==windowBuffer[dimX*dimY/2])? 1:0;
           break;
         }
-        case(ISMAX):{
-          outBuffer[x/down]=(hist.max(windowBuffer)==windowBuffer[dimX*dimY/2])? 1:0;
+        case(filter2d::order):{
+          if(windowBuffer.empty())
+            outBuffer[x/down]=m_noValue;
+          else{
+            double lbound=0;
+            double ubound=dimX*dimY;
+            double theMin=stat.min(windowBuffer);
+            double theMax=stat.max(windowBuffer);
+            double scale=(ubound-lbound)/(theMax-theMin);
+            outBuffer[x/down]=static_cast<short>(scale*(windowBuffer[dimX*dimY/2]-theMin)+lbound);
+          }
           break;
         }
-        case(ORDER):{
-          double lbound=0;
-          double ubound=dimX*dimY;
-          double theMin=hist.min(windowBuffer);
-          double theMax=hist.max(windowBuffer);
-          double scale=(ubound-lbound)/(theMax-theMin);
-          outBuffer[x/down]=static_cast<short>(scale*(windowBuffer[dimX*dimY/2]-theMin)+lbound);
+        case(filter2d::sum):{
+          outBuffer[x/down]=stat.sum(windowBuffer);
           break;
         }
-        case(SUM):{
-          outBuffer[x/down]=hist.sum(windowBuffer);
-          break;
-        }
-        case(HOMOG):
+        case(filter2d::homog):
 	  if(occurrence.size()==1)//all values in window must be the same
 	    outBuffer[x/down]=inBuffer[dimY/2][x];
 	  else//favorize original value in case of ties
 	    outBuffer[x/down]=m_noValue;
           break;
-        case(HETEROG):{
+        case(filter2d::heterog):{
           for(vector<double>::const_iterator wit=windowBuffer.begin();wit!=windowBuffer.end();++wit){
             if(wit==windowBuffer.begin()+windowBuffer.size()/2)
               continue;
@@ -571,17 +606,17 @@ void Filter2d::Filter2d::doit(const ImgReaderGdal& input, ImgWriterGdal& output,
           }
           break;
         }
-        case(DENSITY):{
+        case(filter2d::density):{
 	  if(windowBuffer.size()){
 	    vector<short>::const_iterator vit=m_class.begin();
 	    while(vit!=m_class.end())
 	      outBuffer[x/down]+=100.0*occurrence[*(vit++)]/windowBuffer.size();
 	  }
 	  else
-	    outBuffer[x/down]=0;
+	    outBuffer[x/down]=m_noValue;
           break;
 	}
-        case(MAJORITY):{
+        case(filter2d::majority):{
 	  if(occurrence.size()){
             map<int,int>::const_iterator maxit=occurrence.begin();
             for(map<int,int>::const_iterator mit=occurrence.begin();mit!=occurrence.end();++mit){
@@ -594,10 +629,10 @@ void Filter2d::Filter2d::doit(const ImgReaderGdal& input, ImgWriterGdal& output,
               outBuffer[x/down]=inBuffer[dimY/2][x];
 	  }
 	  else
-	    outBuffer[x/down]=0;
+	    outBuffer[x/down]=m_noValue;
           break;
         }
-        case(THRESHOLD):{
+        case(filter2d::threshold):{
           assert(m_class.size()==m_threshold.size());
 	  if(windowBuffer.size()){
             outBuffer[x/down]=inBuffer[dimY/2][x];//initialize with original value (in case thresholds not met)
@@ -607,10 +642,10 @@ void Filter2d::Filter2d::doit(const ImgReaderGdal& input, ImgWriterGdal& output,
             }
           }
           else
-	    outBuffer[x/down]=0;
+	    outBuffer[x/down]=m_noValue;
           break;
         }
-        case(MIXED):{
+        case(filter2d::mixed):{
           enum Type { BF=11, CF=12, MF=13, NF=20, W=30 };
           double nBF=occurrence[BF];
           double nCF=occurrence[CF];
@@ -656,8 +691,146 @@ void Filter2d::Filter2d::doit(const ImgReaderGdal& input, ImgWriterGdal& output,
   }
 }
 
+void filter2d::Filter2d::mrf(const ImgReaderGdal& input, ImgWriterGdal& output, int dimX, int dimY, double beta, bool eightConnectivity, short down, bool verbose){
+  assert(m_class.size()>1);
+  Vector2d<double> fullBeta(m_class.size(),m_class.size());
+  for(int iclass1=0;iclass1<m_class.size();++iclass1)
+    for(int iclass2=0;iclass2<m_class.size();++iclass2)
+      fullBeta[iclass1][iclass2]=beta;
+  mrf(input,output,dimX,dimY,fullBeta,eightConnectivity,down,verbose);
+}
+
+//beta[classTo][classFrom]
+void filter2d::Filter2d::mrf(const ImgReaderGdal& input, ImgWriterGdal& output, int dimX, int dimY, Vector2d<double> beta, bool eightConnectivity, short down, bool verbose)
+{
+  assert(dimX);
+  assert(dimY);
+  const char* pszMessage;
+  void* pProgressArg=NULL;
+  GDALProgressFunc pfnProgress=GDALTermProgress;
+  double progress=0;
+  pfnProgress(progress,pszMessage,pProgressArg);
+  Vector2d<short> inBuffer(dimY,input.nrOfCol());
+  Vector2d<double> outBuffer(m_class.size(),(input.nrOfCol()+down-1)/down);
+  assert(input.nrOfBand()==1);
+  assert(output.nrOfBand()==m_class.size());
+  assert(m_class.size()>1);
+  assert(beta.size()==m_class.size());
+  //initialize last half of inBuffer
+  int indexI=0;
+  int indexJ=0;
+  for(int j=-dimY/2;j<(dimY+1)/2;++j){
+    try{
+      input.readData(inBuffer[indexJ],GDT_Int16,abs(j));
+    }
+    catch(string errorstring){
+      cerr << errorstring << "in line " << indexJ << endl;
+    }
+    ++indexJ;
+  }
+  for(int y=0;y<input.nrOfRow();++y){
+    if(y){//inBuffer already initialized for y=0
+      //erase first line from inBuffer
+      inBuffer.erase(inBuffer.begin());
+      //read extra line and push back to inBuffer if not out of bounds
+      if(y+dimY/2<input.nrOfRow()){
+        //allocate buffer
+        inBuffer.push_back(inBuffer.back());
+        try{
+          input.readData(inBuffer[inBuffer.size()-1],GDT_Int16,y+dimY/2);
+        }
+        catch(string errorstring){
+          cerr << errorstring << "in line " << y << endl;
+        }
+      }
+      else{
+        int over=y+dimY/2-input.nrOfRow();
+        int index=(inBuffer.size()-1)-over;
+        assert(index>=0);
+        assert(index<inBuffer.size());
+        inBuffer.push_back(inBuffer[index]);
+      }
+    }
+    if((y+1+down/2)%down)
+      continue;
+    for(int x=0;x<input.nrOfCol();++x){
+      if((x+1+down/2)%down)
+        continue;
+      vector<short> potential(m_class.size());
+      for(int iclass=0;iclass<m_class.size();++iclass){
+        potential[iclass]=0;
+        outBuffer[iclass][x/down]=0;
+      }
+      vector<double> windowBuffer;
+      for(int j=-dimY/2;j<(dimY+1)/2;++j){
+        for(int i=-dimX/2;i<(dimX+1)/2;++i){
+          if(i!=0&&j!=0&&!eightConnectivity)
+            continue;
+          if(i==0&&j==0)
+            continue;
+          indexI=x+i;
+          //check if out of bounds
+          if(indexI<0)
+            indexI=-indexI;
+          else if(indexI>=input.nrOfCol())
+            indexI=input.nrOfCol()-i;
+          if(y+j<0)
+            indexJ=-j;
+          else if(y+j>=input.nrOfRow())
+            indexJ=dimY/2-j;
+          else
+            indexJ=dimY/2+j;
+          bool masked=false;
+          for(int imask=0;imask<m_mask.size();++imask){
+            if(inBuffer[indexJ][indexI]==m_mask[imask]){
+              masked=true;
+              break;
+            }
+          }
+          if(!masked){
+            // if(inBuffer[dimY/2][x]//centre pixel
+            for(int iclass=0;iclass<m_class.size();++iclass){
+              if(inBuffer[indexJ][indexI]==m_class[iclass])
+                potential[iclass]+=1;
+            }
+          }
+        }
+      }
+      double norm=0;
+      for(int iclass1=0;iclass1<m_class.size();++iclass1){
+	assert(beta[iclass1].size()==m_class.size());
+        double pot=0;
+        for(int iclass2=0;iclass2<m_class.size();++iclass2)
+	  if(iclass2!=iclass1)
+	    pot+=potential[iclass2]*beta[iclass1][iclass2];
+        double prior=exp(-pot);
+        outBuffer[iclass1][x/down]=prior;
+        norm+=prior;
+      }
+      if(norm){
+        for(int iclass1=0;iclass1<m_class.size();++iclass1)
+          outBuffer[iclass1][x/down]/=norm;
+      }
+    }
+    progress=(1.0+y/down)/output.nrOfRow();
+    pfnProgress(progress,pszMessage,pProgressArg);
+    //write outBuffer to file
+    assert(outBuffer.size()==m_class.size());
+    assert(y<output.nrOfRow());
+    for(int iclass=0;iclass<m_class.size();++iclass){
+      assert(outBuffer[iclass].size()==output.nrOfCol());
+      try{
+        output.writeData(outBuffer[iclass],GDT_Float64,y/down,iclass);
+      }
+      catch(string errorstring){
+        cerr << errorstring << "in class " << iclass << ", line " << y << endl;
+      }
+    }
+  }
+}
+
 //todo: re-implement without dependency of CImg and reg libraries
-// void Filter2d::Filter2d::dwt_texture(const string& inputFilename, const string& outputFilename,int dim, int scale, int down, int iband, bool verbose)
+// void filter2d::Filter2d::dwt_texture(const string& inputFilename, const string& outputFilename,int dim, int scale, int down, int iband, bool verbose)
 // {
 //   ImgReaderGdal input;
 //   ImgWriterGdal output;
@@ -763,11 +936,11 @@ void Filter2d::Filter2d::doit(const ImgReaderGdal& input, ImgWriterGdal& output,
 //   output.close();
 // }
 
-void Filter2d::Filter2d::morphology(const ImgReaderGdal& input, ImgWriterGdal& output, int method, int dimX, int dimY, bool disc, double angle)
+void filter2d::Filter2d::morphology(const ImgReaderGdal& input, ImgWriterGdal& output, const std::string& method, int dimX, int dimY, const vector<double> &angle, bool disc)
 {
   assert(dimX);
   assert(dimY);
-  Histogram hist;
+  statfactory::StatFactory stat;
   for(int iband=0;iband<input.nrOfBand();++iband){
     Vector2d<double> inBuffer(dimY,input.nrOfCol());
     vector<double> outBuffer(input.nrOfCol());
@@ -807,9 +980,8 @@ void Filter2d::Filter2d::morphology(const ImgReaderGdal& input, ImgWriterGdal& o
       for(int x=0;x<input.nrOfCol();++x){
         double currentValue=inBuffer[dimY/2][x];
 	outBuffer[x]=currentValue;
-	vector<double> histBuffer;
+	vector<double> statBuffer;
 	bool currentMasked=false;
-        double rse=0;
 	for(int imask=0;imask<m_mask.size();++imask){
 	  if(currentValue==m_mask[imask]){
 	    currentMasked=true;
@@ -825,42 +997,40 @@ void Filter2d::Filter2d::morphology(const ImgReaderGdal& input, ImgWriterGdal& o
 	      if(disc&&(i*i+j*j>(dimX/2)*(dimY/2)))
 		continue;
               bool masked=false;
-	      if(angle>=-180){
-	      	// double theta;
-	      	// if(i>0)
-	      	//   theta=atan(static_cast<double>(j)/(static_cast<double>(i)));
-	      	// else if(i<0)
-	      	//   theta=PI+atan(static_cast<double>(j)/(static_cast<double>(i)));
-	      	// else if(j>0)
-	      	//   theta=PI/2.0;
-	      	// else if(j<0)
-	      	//   theta=3.0*PI/2.0;
-                // rse=sqrt((theta-DEG2RAD(angle))*(theta-DEG2RAD(angle))/theta/theta);
-                // //test
-                if(angle<45||angle>315)
-                  if((j!=0)||(i>0))//RIGHT
-                    continue;
-                if(angle>135&&angle<225)
-                  if(j!=0||i<0)//LEFT
-                    continue;
-                if(angle>45&&angle<135)
-                  if(j<0||i!=0)//UP
-                    continue;
-                if(angle>225&&angle<315)
-                  if(j>0||i!=0)//DOWN
-                    continue;
-                if(angle>270&&angle<360)
-                  if(j>0||i>0)//LOWER RIGHT
-                    continue;
-                if(angle>0&&angle<90)
-                  if(j<0||i>0)//UPPER RIGHT
-                    continue;
-                if(angle>180&&angle<270)
-                  if(j>0||i<0)//LOWER LEFT
-                    continue;
-                if(angle>90&&angle<180)
-                  if(j<0||i<0)//UPPER LEFT
-                    continue;
+	      if(angle.size()){
+	      	double theta;
+		//use polar coordinates in radians
+	      	if(i>0){
+		  if(j<0)
+		    theta=atan(static_cast<double>(-j)/(static_cast<double>(i)));
+		  else
+		    theta=-atan(static_cast<double>(j)/(static_cast<double>(i)));
+		}
+	      	else if(i<0){
+		  if(j<0)
+		    theta=PI-atan(static_cast<double>(-j)/(static_cast<double>(-i)));
+		  else
+		    theta=PI+atan(static_cast<double>(j)/(static_cast<double>(-i)));
+		}
+	      	else if(j<0)
+	      	  theta=PI/2.0;
+	      	else if(j>0)
+	      	  theta=3.0*PI/2.0;
+		//convert to North (0), East (90), South (180), West (270) in degrees
+		theta=360-(theta/PI*180)+90;
+		if(theta<0)
+		  theta+=360;
+		while(theta>360)
+		  theta-=360;
+		bool alligned=false;
+		for(int iangle=0;iangle<angle.size();++iangle){
+		  if(sqrt((theta-angle[iangle])*(theta-angle[iangle]))<10){
+		    alligned=true;
+		    break;
+		  }
+		}
+		if(!alligned)
+		  continue;
 	      }
 	      indexI=x+i;
 	      //check if out of bounds
@@ -889,23 +1059,23 @@ void Filter2d::Filter2d::morphology(const ImgReaderGdal& input, ImgWriterGdal& o
 		  }
 		}
 		if(m_class.size())
-		  histBuffer.push_back(binValue);
+		  statBuffer.push_back(binValue);
 		else
-		  histBuffer.push_back(inBuffer[indexJ][indexI]);
+		  statBuffer.push_back(inBuffer[indexJ][indexI]);
 	      }
 	    }
           }
-	  if(histBuffer.size()){
-            switch(method){
-            case(DILATE):
-              outBuffer[x]=hist.max(histBuffer);
+	  if(statBuffer.size()){
+            switch(getFilterType(method)){
+            case(filter2d::dilate):
+              outBuffer[x]=stat.max(statBuffer);
               break;
-            case(ERODE):
-              outBuffer[x]=hist.min(histBuffer);
+            case(filter2d::erode):
+              outBuffer[x]=stat.min(statBuffer);
               break;
             default:
               ostringstream ess;
-              ess << "Error:  morphology method " << method << " not supported, choose " << DILATE << " (dilate) or " << ERODE << " (erode)" << endl;
+              ess << "Error:  morphology method " << method << " not supported, choose " << filter2d::dilate << " (dilate) or " << filter2d::erode << " (erode)" << endl;
               throw(ess.str());
               break;
             }
@@ -929,7 +1099,7 @@ void Filter2d::Filter2d::morphology(const ImgReaderGdal& input, ImgWriterGdal& o
   }
 }
 
-void Filter2d::Filter2d::shadowDsm(const ImgReaderGdal& input, ImgWriterGdal& output, double sza, double saa, double pixelSize, short shadowFlag){
+void filter2d::Filter2d::shadowDsm(const ImgReaderGdal& input, ImgWriterGdal& output, double sza, double saa, double pixelSize, short shadowFlag){
   Vector2d<float> inputBuffer;
   Vector2d<float> outputBuffer;
   input.readDataBlock(inputBuffer, GDT_Float32, 0, input.nrOfCol()-1, 0, input.nrOfRow()-1, 0);
