@@ -28,24 +28,10 @@ along with pktools.  If not, see <http://www.gnu.org/licenses/>.
 #include "base/Optionpk.h"
 #include "algorithms/Egcs.h"
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 int main(int argc, char *argv[])
 {
-  std::string versionString="version ";
-  versionString+=VERSION;
-  versionString+=", Copyright (C) 2008-2012 Pieter Kempeneers.\n\
-   This program comes with ABSOLUTELY NO WARRANTY; for details type use option -h.\n\
-   This is free software, and you are welcome to redistribute it\n\
-   under certain conditions; use option --license for details.";
-  Optionpk<bool> version_opt("\0","version",versionString,false);
-  Optionpk<bool> license_opt("lic","license","show license information",false);
-  Optionpk<bool> help_opt("h","help","shows this help info",false);
-  Optionpk<bool> todo_opt("\0","todo","",false);
-  Optionpk<string>  input_opt("i", "input", "Input image file(s). If input contains multiple images, a multi-band output is created", "");
-  Optionpk<string>  output_opt("o", "output", "Output image file", "");
+  Optionpk<string>  input_opt("i", "input", "Input image file(s). If input contains multiple images, a multi-band output is created");
+  Optionpk<string>  output_opt("o", "output", "Output image file");
   Optionpk<string>  projection_opt("p", "projection", "projection in EPSG format (leave blank to copy from input file, use EPSG:3035 to use European projection and to force to European grid", "");
   Optionpk<string>  extent_opt("e", "extent", "get boundary from extent from polygons in vector file", "");
   Optionpk<bool> mask_opt("m","mask","mask values out of polygon in extent file to flag option (tip: for better performance, use gdal_rasterize -i -burn 0 -l extent extent.shp output (with output the result of pkcrop)",false);
@@ -53,60 +39,74 @@ int main(int argc, char *argv[])
   Optionpk<double>  uly_opt("uly", "uly", "Upper left y value bounding box (in geocoordinates if georef is true)", 0.0);
   Optionpk<double>  lrx_opt("lrx", "lrx", "Lower right x value bounding box (in geocoordinates if georef is true)", 0.0);
   Optionpk<double>  lry_opt("lry", "lry", "Lower right y value bounding box (in geocoordinates if georef is true)", 0.0);
-  Optionpk<double>  dx_opt("dx", "dx", "Output resolution in x (in meter) (0.0: keep original resolution)", 0.0);
-  Optionpk<double>  dy_opt("dy", "dy", "Output resolution in y (in meter) (0.0: keep original resolution)", 0.0);
+  Optionpk<double>  dx_opt("dx", "dx", "Output resolution in x (in meter) (0.0: keep original resolution)");
+  Optionpk<double>  dy_opt("dy", "dy", "Output resolution in y (in meter) (0.0: keep original resolution)");
+  Optionpk<double> cx_opt("x", "x", "x-coordinate of image centre to crop (in meter)");
+  Optionpk<double> cy_opt("y", "y", "y-coordinate of image centre to crop (in meter)");
+  Optionpk<double> nx_opt("nx", "nx", "image size in x to crop (in meter)");
+  Optionpk<double> ny_opt("ny", "ny", "image size in y to crop (in meter)");
+  Optionpk<int> ns_opt("ns", "ns", "number of samples  to crop (in pixels)");
+  Optionpk<int> nl_opt("nl", "nl", "number of lines to crop (in pixels)");
   Optionpk<int>  band_opt("b", "band", "band index to crop (-1: crop all bands)", -1);
+  Optionpk<double> autoscale_opt("as", "autoscale", "scale output to min and max, e.g., --autoscale 0 --autoscale 255");
   Optionpk<double> scale_opt("s", "scale", "output=scale*input+offset", 1);
   Optionpk<double> offset_opt("off", "offset", "output=scale*input+offset", 0);
   Optionpk<string>  otype_opt("ot", "otype", "Data type for output image ({Byte/Int16/UInt16/UInt32/Int32/Float32/Float64/CInt16/CInt32/CFloat32/CFloat64}). Empty string: inherit type from input image","");
   Optionpk<string>  oformat_opt("of", "oformat", "Output image format (see also gdal_translate). Empty string: inherit from input image");
-  Optionpk<string>  colorTable_opt("ct", "ct", "color table (file with 5 columns: id R G B ALFA (0: transparent, 255: solid)", "");
+  Optionpk<string>  colorTable_opt("ct", "ct", "color table (file with 5 columns: id R G B ALFA (0: transparent, 255: solid)");
   Optionpk<string> option_opt("co", "co", "options: NAME=VALUE [-co COMPRESS=LZW] [-co INTERLEAVE=BAND]");
   Optionpk<short>  flag_opt("f", "flag", "Flag value to put in image if out of bounds.", 0);
   Optionpk<string>  resample_opt("r", "resampling-method", "Resampling method (near: nearest neighbour, bilinear: bi-linear interpolation).", "near");
   Optionpk<string>  description_opt("d", "description", "Set image description", "");
   Optionpk<bool>  verbose_opt("v", "verbose", "verbose", false);
 
-  version_opt.retrieveOption(argc,argv);
-  license_opt.retrieveOption(argc,argv);
-  help_opt.retrieveOption(argc,argv);
-  todo_opt.retrieveOption(argc,argv);
-
-  input_opt.retrieveOption(argc,argv);
-  output_opt.retrieveOption(argc,argv);
-  projection_opt.retrieveOption(argc,argv);
-  extent_opt.retrieveOption(argc,argv);
-  mask_opt.retrieveOption(argc,argv);
-  ulx_opt.retrieveOption(argc,argv);
-  uly_opt.retrieveOption(argc,argv);
-  lrx_opt.retrieveOption(argc,argv);
-  lry_opt.retrieveOption(argc,argv);
-  band_opt.retrieveOption(argc,argv);
-  scale_opt.retrieveOption(argc,argv);
-  offset_opt.retrieveOption(argc,argv);
-  otype_opt.retrieveOption(argc,argv);
-  oformat_opt.retrieveOption(argc,argv);
-  colorTable_opt.retrieveOption(argc,argv);
-  dx_opt.retrieveOption(argc,argv);
-  dy_opt.retrieveOption(argc,argv);
-  option_opt.retrieveOption(argc,argv);
-  flag_opt.retrieveOption(argc,argv);
-  resample_opt.retrieveOption(argc,argv);
-  description_opt.retrieveOption(argc,argv);
-  verbose_opt.retrieveOption(argc,argv);
-
-  if(version_opt[0]||todo_opt[0]){
-    cout << version_opt.getHelp() << endl;
-    cout << "todo: " << todo_opt.getHelp() << endl;
+  bool doProcess;//stop process when program was invoked with help option (-h --help)
+  try{
+    doProcess=input_opt.retrieveOption(argc,argv);
+    output_opt.retrieveOption(argc,argv);
+    projection_opt.retrieveOption(argc,argv);
+    extent_opt.retrieveOption(argc,argv);
+    mask_opt.retrieveOption(argc,argv);
+    ulx_opt.retrieveOption(argc,argv);
+    uly_opt.retrieveOption(argc,argv);
+    lrx_opt.retrieveOption(argc,argv);
+    lry_opt.retrieveOption(argc,argv);
+    band_opt.retrieveOption(argc,argv);
+    autoscale_opt.retrieveOption(argc,argv);
+    scale_opt.retrieveOption(argc,argv);
+    offset_opt.retrieveOption(argc,argv);
+    otype_opt.retrieveOption(argc,argv);
+    oformat_opt.retrieveOption(argc,argv);
+    colorTable_opt.retrieveOption(argc,argv);
+    dx_opt.retrieveOption(argc,argv);
+    dy_opt.retrieveOption(argc,argv);
+    cx_opt.retrieveOption(argc,argv);
+    cy_opt.retrieveOption(argc,argv);
+    nx_opt.retrieveOption(argc,argv);
+    ny_opt.retrieveOption(argc,argv);
+    ns_opt.retrieveOption(argc,argv);
+    nl_opt.retrieveOption(argc,argv);
+    option_opt.retrieveOption(argc,argv);
+    flag_opt.retrieveOption(argc,argv);
+    resample_opt.retrieveOption(argc,argv);
+    description_opt.retrieveOption(argc,argv);
+    verbose_opt.retrieveOption(argc,argv);
+  }
+  catch(string predefinedString){
+    std::cout << predefinedString << std::endl;
     exit(0);
   }
-  if(license_opt[0]){
-    cout << Optionpk<bool>::getGPLv3License() << endl;
-    exit(0);
+  if(!doProcess){
+    std::cout << "short option -h shows basic options only, use long option --help to show all options" << std::endl;
+    exit(0);//help was invoked, stop processing
   }
-  if(help_opt[0]){
-    cout << "usage: pkcrop -i inputimage -o outputimage [OPTIONS]" << endl;
-    exit(0);
+  if(input_opt.empty()){
+    std::cerr << "No input file provided (use option -i). Use pkinfo --help for help information" << std::endl;
+    exit(0);//help was invoked, stop processing
+  }
+  if(output_opt.empty()){
+    std::cerr << "No output file provided (use option -i). Use pkinfo --help for help information" << std::endl;
+    exit(0);//help was invoked, stop processing
   }
 
   RESAMPLE theResample;
@@ -132,6 +132,27 @@ int main(int argc, char *argv[])
   pfnProgress(progress,pszMessage,pProgressArg);
   ImgReaderGdal imgReader;
   ImgWriterGdal imgWriter;
+  //open input images to extract number of bands and spatial resolution
+  int ncropband=0;//total number of bands to write
+  double dx=(dx_opt.size())? dx_opt[0]:0;
+  double dy=(dy_opt.size())? dy_opt[0]:0;
+  for(int iimg=0;iimg<input_opt.size();++iimg){
+    imgReader.open(input_opt[iimg]);
+    if(dx_opt.empty()){
+      if(!iimg||imgReader.getDeltaX()<dx)
+        dx=imgReader.getDeltaX();
+    }
+    if(dy_opt.empty()){
+      if(!iimg||imgReader.getDeltaY()<dy)
+        dy=imgReader.getDeltaY();
+    }
+    if(band_opt[0]>=0)
+      ncropband+=band_opt.size();
+    else
+      ncropband+=imgReader.nrOfBand();
+    imgReader.close();
+  }
+
   GDALDataType theType=GDT_Unknown;
   if(verbose_opt[0])
     cout << "possible output data types: ";
@@ -150,8 +171,6 @@ int main(int argc, char *argv[])
     else
       cout << "Output pixel type:  " << GDALGetDataTypeName(theType) << endl;
   }
-  double dx=dx_opt[0];
-  double dy=dy_opt[0];
   //bounding box of cropped image
   double cropulx=ulx_opt[0];
   double cropuly=uly_opt[0];
@@ -166,38 +185,48 @@ int main(int argc, char *argv[])
         cerr << "Error: could not get extent from " << extent_opt[0] << endl;
         exit(1);
       }
-      if(ulx_opt[0]<cropulx)
-        cropulx=ulx_opt[0];
-      if(uly_opt[0]>cropuly)
-        cropuly=uly_opt[0];
-      if(lry_opt[0]<croplry)
-        croplry=lry_opt[0];
-      if(lrx_opt[0]>croplrx)
-        croplrx=lrx_opt[0];
       extentReader.close();
     }
     if(mask_opt[0])
       extentReader.open(extent_opt[0]);
   }
+  else if(cx_opt.size()&&cy_opt.size()&&nx_opt.size()&&ny_opt.size()){
+    ulx_opt[0]=cx_opt[0]-nx_opt[0]/2.0;
+    uly_opt[0]=cy_opt[0]+ny_opt[0]/2.0;
+    lrx_opt[0]=cx_opt[0]+nx_opt[0]/2.0;
+    lry_opt[0]=cy_opt[0]-ny_opt[0]/2.0;
+  }
+  else if(cx_opt.size()&&cy_opt.size()&&ns_opt.size()&&nl_opt.size()){
+    ulx_opt[0]=cx_opt[0]-ns_opt[0]*dx/2.0;
+    uly_opt[0]=cy_opt[0]+nl_opt[0]*dy/2.0;
+    lrx_opt[0]=cx_opt[0]+ns_opt[0]*dx/2.0;
+    lry_opt[0]=cy_opt[0]-nl_opt[0]*dy/2.0;
+  }
+  if(ulx_opt[0]<cropulx)
+    cropulx=ulx_opt[0];
+  if(uly_opt[0]>cropuly)
+    cropuly=uly_opt[0];
+  if(lry_opt[0]<croplry)
+    croplry=lry_opt[0];
+  if(lrx_opt[0]>croplrx)
+    croplrx=lrx_opt[0];
   if(verbose_opt[0])
     cout << "--ulx=" << ulx_opt[0] << " --uly=" << uly_opt[0] << " --lrx=" << lrx_opt[0] << " --lry=" << lry_opt[0] << endl;
   //determine number of output bands
-  int ncropband=0;//total number of bands to write
   int writeBand=0;//write band
 
   while(scale_opt.size()<band_opt.size())
     scale_opt.push_back(scale_opt[0]);
   while(offset_opt.size()<band_opt.size())
     offset_opt.push_back(offset_opt[0]);
-
-  for(int iimg=0;iimg<input_opt.size();++iimg){
-    imgReader.open(input_opt[iimg]);
-    if(band_opt[0]>=0)
-      ncropband+=band_opt.size();
-    else
-      ncropband+=imgReader.nrOfBand();
-    imgReader.close();
+  if(autoscale_opt.size()){
+    assert(autoscale_opt.size()%2==0);
+    while(autoscale_opt.size()<band_opt.size()*2){
+      autoscale_opt.push_back(autoscale_opt[0]);
+      autoscale_opt.push_back(autoscale_opt[1]);
+    }
   }
+
   for(int iimg=0;iimg<input_opt.size();++iimg){
     if(verbose_opt[0])
       cout << "opening image " << input_opt[iimg] << endl;
@@ -217,10 +246,10 @@ int main(int argc, char *argv[])
     int ncol=imgReader.nrOfCol();
     int ncropcol=0;
     int ncroprow=0;
-    if(!dx||!dy){
-      dx=imgReader.getDeltaX();
-      dy=imgReader.getDeltaY();
-    }
+    // if(!dx||!dy){
+    //   dx=imgReader.getDeltaX();
+    //   dy=imgReader.getDeltaY();
+    // }
     if(verbose_opt[0])
       cout << "size of " << input_opt[iimg] << ": " << ncol << " cols, "<< nrow << " rows" << endl;
     double uli,ulj,lri,lrj;//image coordinates
@@ -327,17 +356,12 @@ int main(int argc, char *argv[])
       }
       else if(imgReader.isGeoRef())
 	imgWriter.setProjection(imgReader.getProjection());
-      if(colorTable_opt[0]!=""){
-        if(verbose_opt[0])
-          cout << "set colortable " << colorTable_opt[0] << endl;
-        assert(imgWriter.getDataType()==GDT_Byte);
-        imgWriter.setColorTable(colorTable_opt[0]);
+      if(colorTable_opt.size()){
+        if(colorTable_opt[0]!="none")
+          imgWriter.setColorTable(colorTable_opt[0]);
       }
-      else if(imgReader.getColorTable()!=NULL){
-        if(verbose_opt[0])
-          cout << "set colortable from input image" << endl;
+      else if (imgReader.getColorTable()!=NULL)//copy colorTable from input image
         imgWriter.setColorTable(imgReader.getColorTable());
-      }
     }
     double startCol=uli;
     double endCol=lri;
@@ -349,6 +373,17 @@ int main(int argc, char *argv[])
       endCol=0;
     else if(lri>=imgReader.nrOfCol())
       endCol=imgReader.nrOfCol()-1;
+    double startRow=ulj;
+    double endRow=lrj;
+    if(ulj<0)
+      startRow=0;
+    else if(ulj>=imgReader.nrOfRow())
+      startRow=imgReader.nrOfRow()-1;
+    if(lrj<0)
+      endRow=0;
+    else if(lrj>=imgReader.nrOfRow())
+      endRow=imgReader.nrOfRow()-1;
+
     int readncol=endCol-startCol+1;
     vector<double> readBuffer(readncol+1);
     int nband=(band_opt[0]<0)?imgReader.nrOfBand():band_opt.size();
@@ -358,6 +393,10 @@ int main(int argc, char *argv[])
 	cout << "extracting band " << readBand << endl;
 	pfnProgress(progress,pszMessage,pProgressArg);
       }
+      double theMin=0;
+      double theMax=0;
+      if(autoscale_opt.size())
+	imgReader.getMinMax(static_cast<int>(startCol),static_cast<int>(endCol),static_cast<int>(startRow),static_cast<int>(endRow),readBand,theMin,theMax);
       double readRow=0;
       double readCol=0;
       double lowerCol=0;
@@ -427,8 +466,18 @@ int main(int argc, char *argv[])
                 if(!valid)
                   writeBuffer.push_back(flag_opt[0]);
                 else{
-                  double theScale=(scale_opt.size()>1)?scale_opt[iband]:scale_opt[0];
-                  double theOffset=(offset_opt.size()>1)?offset_opt[iband]:offset_opt[0];
+                  double theScale=1;
+                  double theOffset=0;
+		  if(autoscale_opt.size()){
+		    theScale=(autoscale_opt[1]-autoscale_opt[0])/(theMax-theMin);
+		    theOffset=autoscale_opt[0]-theScale*theMin;
+		    //scale=(ubound-lbound)/(max-min)
+		    //output=scale*(input-min)+lbound
+		  }
+		  else{
+		    theScale=(scale_opt.size()>1)?scale_opt[iband]:scale_opt[0];
+		    theOffset=(offset_opt.size()>1)?offset_opt[iband]:offset_opt[0];
+		  }
                   switch(theResample){
                   case(BILINEAR):
                     lowerCol=readCol-0.5;
@@ -483,7 +532,7 @@ int main(int argc, char *argv[])
     }
     imgReader.close();
   }
-  if(extent_opt[0]!=""){
+  if(extent_opt[0]!=""&&mask_opt[0]){
     extentReader.close();
   }
   imgWriter.close();
