@@ -25,22 +25,8 @@ along with pktools.  If not, see <http://www.gnu.org/licenses/>.
 #include "Optionpk.h"
 #include "algorithms/ConfusionMatrix.h"
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 int main(int argc, char *argv[])
 {
-  std::string versionString="version ";
-  versionString+=VERSION;
-  versionString+=", Copyright (C) 2008-2012 Pieter Kempeneers.\n\
-   This program comes with ABSOLUTELY NO WARRANTY; for details type use option -h.\n\
-   This is free software, and you are welcome to redistribute it\n\
-   under certain conditions; use option --license for details.";
-  Optionpk<bool> version_opt("\0","version",versionString,false);
-  Optionpk<bool> license_opt("lic","license","show license information",false);
-  Optionpk<bool> help_opt("h","help","shows this help info",false);
-  Optionpk<bool> todo_opt("\0","todo","todo: support different data types (now only integer type supported)",false);
   Optionpk<string> input_opt("i", "input", "Input image file.", "");
   Optionpk<string> reference_opt("r", "reference", "Reference image file", "");
   Optionpk<string> output_opt("o", "output", "Output image file. Default is empty: no output image, only report difference or identical.", "");
@@ -63,45 +49,37 @@ int main(int argc, char *argv[])
   Optionpk<string> option_opt("co", "co", "options: NAME=VALUE [-co COMPRESS=LZW] [-co INTERLEAVE=BAND]");
   Optionpk<short> verbose_opt("v", "verbose", "verbose (default value is 0)", 0);
 
-  version_opt.retrieveOption(argc,argv);
-  license_opt.retrieveOption(argc,argv);
-  help_opt.retrieveOption(argc,argv);
-  todo_opt.retrieveOption(argc,argv);
-
-  if(version_opt[0]||todo_opt[0]){
-    cout << version_opt.getHelp() << endl;
-    cout << "todo: " << todo_opt.getHelp() << endl;
+  bool doProcess;//stop process when program was invoked with help option (-h --help)
+  try{
+    doProcess=input_opt.retrieveOption(argc,argv);
+    output_opt.retrieveOption(argc,argv);
+    option_opt.retrieveOption(argc,argv);
+    reference_opt.retrieveOption(argc,argv);
+    mask_opt.retrieveOption(argc,argv);
+    colorTable_opt.retrieveOption(argc,argv);
+    valueE_opt.retrieveOption(argc,argv);
+    valueO_opt.retrieveOption(argc,argv);
+    valueC_opt.retrieveOption(argc,argv);
+    flag_opt.retrieveOption(argc,argv);
+    mflag_opt.retrieveOption(argc,argv);
+    band_opt.retrieveOption(argc,argv);
+    confusion_opt.retrieveOption(argc,argv);
+    lzw_opt.retrieveOption(argc,argv);
+    labelref_opt.retrieveOption(argc,argv);
+    labelclass_opt.retrieveOption(argc,argv);
+    class_opt.retrieveOption(argc,argv);
+    boundary_opt.retrieveOption(argc,argv);
+    disc_opt.retrieveOption(argc,argv);
+    homogeneous_opt.retrieveOption(argc,argv);
+    verbose_opt.retrieveOption(argc,argv);
+  }
+  catch(string predefinedString){
+    std::cout << predefinedString << std::endl;
     exit(0);
   }
-  if(license_opt[0]){
-    cout << Optionpk<bool>::getGPLv3License() << endl;
-    exit(0);
-  }
-  input_opt.retrieveOption(argc,argv);
-  output_opt.retrieveOption(argc,argv);
-  option_opt.retrieveOption(argc,argv);
-  reference_opt.retrieveOption(argc,argv);
-  mask_opt.retrieveOption(argc,argv);
-  colorTable_opt.retrieveOption(argc,argv);
-  valueE_opt.retrieveOption(argc,argv);
-  valueO_opt.retrieveOption(argc,argv);
-  valueC_opt.retrieveOption(argc,argv);
-  flag_opt.retrieveOption(argc,argv);
-  mflag_opt.retrieveOption(argc,argv);
-  band_opt.retrieveOption(argc,argv);
-  confusion_opt.retrieveOption(argc,argv);
-  lzw_opt.retrieveOption(argc,argv);
-  labelref_opt.retrieveOption(argc,argv);
-  labelclass_opt.retrieveOption(argc,argv);
-  class_opt.retrieveOption(argc,argv);
-  boundary_opt.retrieveOption(argc,argv);
-  disc_opt.retrieveOption(argc,argv);
-  homogeneous_opt.retrieveOption(argc,argv);
-  verbose_opt.retrieveOption(argc,argv);
-
-  if(help_opt[0]){
-    cout << "usage: pkdiff -i inputimage -r referenceimage [OPTIONS]" << endl;
-    exit(0);
+  if(!doProcess){
+    std::cout << "short option -h shows basic options only, use long option --help to show all options" << std::endl;
+    exit(0);//help was invoked, stop processing
   }
 
   ImgReaderGdal inputReader;
@@ -428,8 +406,12 @@ int main(int argc, char *argv[])
                       ++ntotalValidation;
                       int rc=distance(referenceRange.begin(),find(referenceRange.begin(),referenceRange.end(),referenceValue));
                       int ic=distance(inputRange.begin(),find(inputRange.begin(),inputRange.end(),inputValue));
-                      assert(rc<nclass);
-                      assert(ic<nclass);
+                      if(rc>=nclass)
+                        continue;
+                      if(ic>=nclass)
+                        continue;
+                      // assert(rc<nclass);
+                      // assert(ic<nclass);
                       ++nvalidation[rc];
                       ++resultClass[rc][ic];
                       if(verbose_opt[0]>1)
