@@ -24,117 +24,94 @@ along with pktools.  If not, see <http://www.gnu.org/licenses/>.
 #include "imageclasses/ImgReaderGdal.h"
 #include "imageclasses/ImgReaderOgr.h"
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 int main(int argc, char *argv[])
 {
-  std::string versionString="version ";
-  versionString+=VERSION;
-  versionString+=", Copyright (C) 2008-2012 Pieter Kempeneers.\n\
-   This program comes with ABSOLUTELY NO WARRANTY; for details type use option -h.\n\
-   This is free software, and you are welcome to redistribute it\n\
-   under certain conditions; use option --license for details.";
-  Optionpk<bool> version_opt("\0","version",versionString,false);
-  Optionpk<bool> license_opt("lic","license","show license information",false);
-  Optionpk<bool> help_opt("h","help","shows this help info",false);
-  Optionpk<bool> todo_opt("\0","todo","",false);
-  Optionpk<std::string> input_opt("i","input","input image file","");
-  Optionpk<bool>  bbox_opt("bb", "bbox", "show bounding box ", false);
-  Optionpk<bool>  bbox_te_opt("te", "te", "show bounding box in GDAL format: xmin ymin xmax ymax ", false);
-  Optionpk<bool>  centre_opt("c", "centre", "Image centre in projected X,Y coordinates ", false);
-  Optionpk<bool>  colorTable_opt("ct", "colourtable", "show colour table ", false);
-  Optionpk<bool>  samples_opt("s", "samples", "Number of samples in image ", false);
-  Optionpk<bool>  lines_opt("l", "lines", "Number of lines in image ", false);
-  Optionpk<bool>  nband_opt("nb", "nband", "Show number of bands in image", false);
-  Optionpk<short>  band_opt("b", "band", "Band specific information", 0);
-  Optionpk<bool>  dx_opt("dx", "dx", "get resolution in x (in m)", false);
-  Optionpk<bool>  dy_opt("dy", "dy", "get resolution in y (in m)", false);
-  Optionpk<bool>  minmax_opt("mm", "minmax", "Show min and max value of the image ", false);
-  Optionpk<bool>  stat_opt("stat", "stat", "Show statistics (min,max, mean and stdDev of the image ", false);
-  Optionpk<double>  min_opt("m", "min", "Show min value of the image (or set minimum for histogram)", 0);
-  Optionpk<double>  max_opt("M", "max", "Show max value of the image (or set maximum for histogram)", 0);
-  Optionpk<bool>  relative_opt("rel", "rel", "Calculate relative histogram in percentage", false);
-  Optionpk<bool>  projection_opt("p", "projection", "Show projection of the image ", false);
-  Optionpk<bool>  geo_opt("geo", "geo", "get geotransform:  ", false);
-  Optionpk<bool>  interleave_opt("il", "interleave", "Show interleave ", false);
-  Optionpk<bool>  filename_opt("f", "filename", "Image filename ", false);
-  Optionpk<bool>  cover_opt("cov", "cover", "Image covers bounding box (or x and y pos) if printed to std out ", false);
-  Optionpk<double>  x_opt("x", "xpos", "x pos", -1);
-  Optionpk<double>  y_opt("y", "ypos", "y pos", -1);
-  Optionpk<bool>  read_opt("r", "read", "read row y (in projected coordinates if geo option is set, otherwise in image coordinates, 0 based)", 0);
-  Optionpk<bool>  refpixel_opt("ref", "ref", "get reference pixel (lower left corner of centre of gravity pixel)", false);
-  Optionpk<bool>  driver_opt("of", "oformat", "get driver description ", false);
-  Optionpk<std::string>  extent_opt("e", "extent", "get boundary from extent from polygons in vector file", "");
-  Optionpk<double>  ulx_opt("ulx", "ulx", "Upper left x value bounding box (0)", 0.0);
-  Optionpk<double>  uly_opt("uly", "uly", "Upper left y value bounding box (0)", 0.0);
-  Optionpk<double>  lrx_opt("lrx", "lrx", "Lower right x value bounding box (0)", 0.0);
-  Optionpk<double>  lry_opt("lry", "lry", "Lower right y value bounding box (0)", 0.0);
-  Optionpk<bool>  hist_opt("hist", "hist", "Calculate histogram. Use --rel for a relative histogram output. ", false);
-  Optionpk<short>  nbin_opt("nbin", "nbin", "Number of bins used in histogram. Use 0 for all input values as integers", 0);
-  Optionpk<bool>  type_opt("ot", "otype", "Return data type", false);
-  Optionpk<bool>  description_opt("d", "description", "Return image description", false);
-  Optionpk<bool>  metadata_opt("meta", "meta", "Show meta data ", false);
-  Optionpk<double> maskValue_opt("mask", "mask", "mask value(s) for no data to calculate reference pixel in image",0);
-  
-  version_opt.retrieveOption(argc,argv);
-  license_opt.retrieveOption(argc,argv);
-  help_opt.retrieveOption(argc,argv);
-  todo_opt.retrieveOption(argc,argv);
+  Optionpk<std::string> input_opt("i","input","Input image file");
+  Optionpk<bool>  bbox_opt("bb", "bbox", "Shows bounding box ", false,0);
+  Optionpk<bool>  bbox_te_opt("te", "te", "Shows bounding box in GDAL format: xmin ymin xmax ymax ", false,0);
+  Optionpk<bool>  centre_opt("c", "centre", "Image centre in projected X,Y coordinates ", false,0);
+  Optionpk<bool>  colorTable_opt("ct", "colourtable", "Shows colour table ", false,0);
+  Optionpk<bool>  samples_opt("ns", "ns", "Number of samples in image ", false,0);
+  Optionpk<bool>  lines_opt("nl", "nl", "Number of lines in image ", false,0);
+  Optionpk<bool>  nband_opt("nb", "nband", "Show number of bands in image", false,0);
+  Optionpk<short>  band_opt("b", "band", "Band specific information", 0,0);
+  Optionpk<bool>  dx_opt("dx", "dx", "Gets resolution in x (in m)", false,0);
+  Optionpk<bool>  dy_opt("dy", "dy", "Gets resolution in y (in m)", false,0);
+  Optionpk<bool>  minmax_opt("mm", "minmax", "Shows min and max value of the image ", false,0);
+  Optionpk<bool>  stat_opt("stat", "stat", "Shows statistics (min,max, mean and stdDev of the image)", false,0);
+  Optionpk<double>  min_opt("min", "min", "Sets minimum for histogram");
+  Optionpk<double>  max_opt("max", "max", "Sets maximum for histogram");
+  Optionpk<bool>  relative_opt("rel", "rel", "Calculates relative histogram in percentage", false,0);
+  Optionpk<bool>  projection_opt("p", "projection", "Shows projection of the image ", false,0);
+  Optionpk<bool>  geo_opt("geo", "geo", "Gets geotransform  ", false,0);
+  Optionpk<bool>  interleave_opt("il", "interleave", "Shows interleave ", false,0);
+  Optionpk<bool>  filename_opt("f", "filename", "Shows image filename ", false,0);
+  Optionpk<bool>  cover_opt("cov", "cover", "Image covers bounding box (or x and y pos) if printed to std out ", false,0);
+  Optionpk<double>  x_opt("x", "xpos", "x pos");
+  Optionpk<double>  y_opt("y", "ypos", "y pos");
+  Optionpk<bool>  read_opt("r", "read", "Reads row y (in projected coordinates if geo option is set, otherwise in image coordinates, 0 based)",false,0);
+  Optionpk<bool>  refpixel_opt("ref", "ref", "Gets reference pixel (lower left corner of centre of gravity pixel)", false,0);
+  Optionpk<bool>  driver_opt("of", "oformat", "Gets driver description ", false,0);
+  Optionpk<std::string>  extent_opt("e", "extent", "Gets boundary from extent from polygons in vector file");
+  Optionpk<double>  ulx_opt("ulx", "ulx", "Upper left x value bounding box");
+  Optionpk<double>  uly_opt("uly", "uly", "Upper left y value bounding box");
+  Optionpk<double>  lrx_opt("lrx", "lrx", "Lower right x value bounding box");
+  Optionpk<double>  lry_opt("lry", "lry", "Lower right y value bounding box");
+  Optionpk<bool>  hist_opt("hist", "hist", "Calculates histogram. Use --rel for a relative histogram output. ", false,0);
+  Optionpk<short>  nbin_opt("nbin", "nbin", "Number of bins used in histogram. Use 0 for all input values as integers", 0,0);
+  Optionpk<bool>  type_opt("ot", "otype", "Returns data type", false,0);
+  Optionpk<bool>  description_opt("d", "description", "Returns image description", false,0);
+  Optionpk<bool>  metadata_opt("meta", "meta", "Shows meta data ", false,0);
+  Optionpk<double> nodata_opt("nodata", "nodata", "Sets no data value(s) for calculations (flags in input image)");
 
-  if(version_opt[0]){
-    std::cout << version_opt.getHelp() << std::endl;
+  bool doProcess;//stop process when program was invoked with help option (-h --help)
+  try{
+    doProcess=input_opt.retrieveOption(argc,argv);
+    bbox_opt.retrieveOption(argc,argv);
+    bbox_te_opt.retrieveOption(argc,argv);
+    centre_opt.retrieveOption(argc,argv);
+    colorTable_opt.retrieveOption(argc,argv);
+    samples_opt.retrieveOption(argc,argv);
+    lines_opt.retrieveOption(argc,argv);
+    nband_opt.retrieveOption(argc,argv);
+    band_opt.retrieveOption(argc,argv);
+    dx_opt.retrieveOption(argc,argv);
+    dy_opt.retrieveOption(argc,argv);
+    minmax_opt.retrieveOption(argc,argv);
+    stat_opt.retrieveOption(argc,argv);
+    min_opt.retrieveOption(argc,argv);
+    max_opt.retrieveOption(argc,argv);
+    relative_opt.retrieveOption(argc,argv);
+    projection_opt.retrieveOption(argc,argv);
+    geo_opt.retrieveOption(argc,argv);
+    interleave_opt.retrieveOption(argc,argv);
+    filename_opt.retrieveOption(argc,argv);
+    cover_opt.retrieveOption(argc,argv);
+    x_opt.retrieveOption(argc,argv);
+    y_opt.retrieveOption(argc,argv);
+    read_opt.retrieveOption(argc,argv);
+    refpixel_opt.retrieveOption(argc,argv);
+    driver_opt.retrieveOption(argc,argv);
+    extent_opt.retrieveOption(argc,argv);
+    ulx_opt.retrieveOption(argc,argv);
+    uly_opt.retrieveOption(argc,argv);
+    lrx_opt.retrieveOption(argc,argv);
+    lry_opt.retrieveOption(argc,argv);
+    hist_opt.retrieveOption(argc,argv);
+    nbin_opt.retrieveOption(argc,argv);
+    type_opt.retrieveOption(argc,argv);
+    description_opt.retrieveOption(argc,argv);
+    metadata_opt.retrieveOption(argc,argv);
+    nodata_opt.retrieveOption(argc,argv);
+  }
+  catch(string predefinedString){
+    std::cout << predefinedString << std::endl;
     exit(0);
   }
-  if(license_opt[0]){
-    cout << Optionpk<bool>::getGPLv3License() << endl;
-    exit(0);
+  if(!doProcess){
+    std::cout << "short option -h shows basic options only, use long option --help to show all options" << std::endl;
+    exit(0);//help was invoked, stop processing
   }
-
-  input_opt.retrieveOption(argc,argv);
-  bbox_opt.retrieveOption(argc,argv);
-  bbox_te_opt.retrieveOption(argc,argv);
-  centre_opt.retrieveOption(argc,argv);
-  colorTable_opt.retrieveOption(argc,argv);
-  samples_opt.retrieveOption(argc,argv);
-  lines_opt.retrieveOption(argc,argv);
-  nband_opt.retrieveOption(argc,argv);
-  band_opt.retrieveOption(argc,argv);
-  dx_opt.retrieveOption(argc,argv);
-  dy_opt.retrieveOption(argc,argv);
-  minmax_opt.retrieveOption(argc,argv);
-  stat_opt.retrieveOption(argc,argv);
-  min_opt.retrieveOption(argc,argv);
-  max_opt.retrieveOption(argc,argv);
-  relative_opt.retrieveOption(argc,argv);
-  projection_opt.retrieveOption(argc,argv);
-  geo_opt.retrieveOption(argc,argv);
-  interleave_opt.retrieveOption(argc,argv);
-  filename_opt.retrieveOption(argc,argv);
-  cover_opt.retrieveOption(argc,argv);
-  x_opt.retrieveOption(argc,argv);
-  y_opt.retrieveOption(argc,argv);
-  read_opt.retrieveOption(argc,argv);
-  refpixel_opt.retrieveOption(argc,argv);
-  driver_opt.retrieveOption(argc,argv);
-  extent_opt.retrieveOption(argc,argv);
-  ulx_opt.retrieveOption(argc,argv);
-  uly_opt.retrieveOption(argc,argv);
-  lrx_opt.retrieveOption(argc,argv);
-  lry_opt.retrieveOption(argc,argv);
-  hist_opt.retrieveOption(argc,argv);
-  nbin_opt.retrieveOption(argc,argv);
-  type_opt.retrieveOption(argc,argv);
-  description_opt.retrieveOption(argc,argv);
-  metadata_opt.retrieveOption(argc,argv);
-  maskValue_opt.retrieveOption(argc,argv);
-
-  if(help_opt[0]){
-    std::cout << "usage: pkinfo -i imagefile [OPTIONS]" << std::endl;
-    exit(0);
-  }
-
   //for union
   double maxLRX=0;
   double maxULY=0;
@@ -150,9 +127,11 @@ int main(int argc, char *argv[])
   ImgReaderGdal imgReader;
   for(int ifile=0;ifile<input_opt.size();++ifile){
     imgReader.open(input_opt[ifile]);
-    for(int inodata=0;inodata<maskValue_opt.size();++inodata)
-      imgReader.pushNoDataValue(maskValue_opt[inodata],band_opt[0]);
-
+    for(int inodata=0;inodata<nodata_opt.size();++inodata){
+      if(!inodata)
+        imgReader.GDALSetNoDataValue(nodata_opt[0],band_opt[0]);//only single no data can be set in GDALRasterBand (used for ComputeStatistics)
+      imgReader.pushNoDataValue(nodata_opt[inodata]);
+    }
     if(filename_opt[0])
       std::cout << " --input " << input_opt[ifile] << " ";
     if(centre_opt[0]){
@@ -209,7 +188,7 @@ int main(int argc, char *argv[])
     if(cover_opt[0]){
       //get bounding box from extentReader if defined
       ImgReaderOgr extentReader;
-      if(extent_opt[0]!=""){
+      if(extent_opt.size()){
         extentReader.open(extent_opt[0]);
         if(!(extentReader.getExtent(ulx_opt[0],uly_opt[0],lrx_opt[0],lry_opt[0]))){
           cerr << "Error: could not get extent from " << extent_opt[0] << std::endl;
@@ -219,35 +198,39 @@ int main(int argc, char *argv[])
       }
       double theULX, theULY, theLRX, theLRY;
       imgReader.getBoundingBox(theULX,theULY,theLRX,theLRY);
-      if((ulx_opt[0]||uly_opt[0]||lrx_opt[0]||lry_opt[0])&&(imgReader.covers(ulx_opt[0],uly_opt[0],lrx_opt[0],lry_opt[0])))
+      if((ulx_opt.size()||uly_opt.size()||lrx_opt.size()||lry_opt.size())&&(imgReader.covers(ulx_opt[0],uly_opt[0],lrx_opt[0],lry_opt[0])))
 	std::cout << " -i " << input_opt[ifile] << " ";
       else if(imgReader.covers(x_opt[0],y_opt[0]))
 	std::cout << " -i " << input_opt[ifile] << " ";
 
     }
-    else if(ulx_opt[0]||uly_opt[0]||lrx_opt[0]||lry_opt[0]){
+    else if(ulx_opt.size()||uly_opt.size()||lrx_opt.size()||lry_opt.size()){
       double ulx,uly,lrx,lry;
       imgReader.getBoundingBox(ulx,uly,lrx,lry);
-      if(ulx_opt[0])
+      if(ulx_opt.size())
         std::cout << " --ulx=" << fixed << ulx << " ";
-      if(uly_opt[0])
+      if(uly_opt.size())
         std::cout << " --uly=" << fixed << uly << " ";
-      if(lrx_opt[0])
+      if(lrx_opt.size())
         std::cout << " --lrx=" << fixed << lrx << " ";
-      if(lry_opt[0])
+      if(lry_opt.size())
         std::cout << " --lry=" << fixed << lry << " ";
     }
     if(colorTable_opt[0]){
       GDALColorTable* colorTable=imgReader.getColorTable();
-      for(int index=0;index<colorTable->GetColorEntryCount();++index){
-        GDALColorEntry sEntry=*(colorTable->GetColorEntry(index));
-        std::cout << index << " " << sEntry.c1 << " " << sEntry.c2 << " " << sEntry.c3 << " " << sEntry.c4 << std::endl;
+      if(colorTable!=NULL){
+        for(int index=0;index<colorTable->GetColorEntryCount();++index){
+          GDALColorEntry sEntry=*(colorTable->GetColorEntry(index));
+          std::cout << index << " " << sEntry.c1 << " " << sEntry.c2 << " " << sEntry.c3 << " " << sEntry.c4 << std::endl;
+        }
       }
+      else
+        std::cout << "--ct none ";
     }
     if(samples_opt[0])
-      std::cout << "--samples " << imgReader.nrOfCol() << " ";
+      std::cout << "--ns " << imgReader.nrOfCol() << " ";
     if(lines_opt[0])
-      std::cout << "--rows " << imgReader.nrOfRow() << " ";
+      std::cout << "--nl " << imgReader.nrOfRow() << " ";
     if(nband_opt[0])
       std::cout << "--nband " << imgReader.nrOfBand() << " ";
     double minValue=0;
@@ -266,17 +249,23 @@ int main(int argc, char *argv[])
 
     if(minmax_opt[0]){
       assert(band_opt[0]<imgReader.nrOfBand());
-      imgReader.getMinMax(minValue,maxValue,band_opt[0],true);
+      if((ulx_opt.size()||uly_opt.size()||lrx_opt.size()||lry_opt.size())&&(imgReader.covers(ulx_opt[0],uly_opt[0],lrx_opt[0],lry_opt[0]))){
+	double uli,ulj,lri,lrj;
+	imgReader.geo2image(ulx_opt[0],uly_opt[0],uli,ulj);
+	imgReader.geo2image(lrx_opt[0],lry_opt[0],lri,lrj);
+	imgReader.getMinMax(static_cast<int>(uli),static_cast<int>(lri),static_cast<int>(ulj),static_cast<int>(lrj),band_opt[0],minValue,maxValue);
+      }
+      else
+	imgReader.getMinMax(minValue,maxValue,band_opt[0],true);
       std::cout << "--min " << minValue << " --max " << maxValue << " ";
     }
     if(hist_opt[0]){
       assert(band_opt[0]<imgReader.nrOfBand());
-      if(min_opt[0]==max_opt[0])
-        imgReader.getMinMax(minValue,maxValue,band_opt[0]);
-      else{
+      imgReader.getMinMax(minValue,maxValue,band_opt[0]);
+      if(min_opt.size())
         minValue=min_opt[0];
+      if(max_opt.size())
         maxValue=max_opt[0];
-      }
       int nbin=nbin_opt[0];
       if(nbin_opt[0]==0)
 	nbin=maxValue-minValue+1;
@@ -289,14 +278,19 @@ int main(int argc, char *argv[])
       for(int irow=0;irow<imgReader.nrOfRow();++irow){
 	imgReader.readData(lineBuffer,GDT_Float64,irow,band_opt[0]);
 	for(int icol=0;icol<imgReader.nrOfCol();++icol){
-	  if(lineBuffer[icol]>maxValue)
+          if(imgReader.isNoData(lineBuffer[icol]))
+            ++ninvalid;
+          else if(lineBuffer[icol]>maxValue)
             ++ninvalid;
           else if(lineBuffer[icol]<minValue)
             ++ninvalid;
 	  else if(lineBuffer[icol]==maxValue)
 	    ++output[nbin-1];
-	  else if(static_cast<double>(lineBuffer[icol]-minValue)/(maxValue-minValue)*nbin>=nbin)
-	    ++output[nbin-1];
+	  // else if(static_cast<double>(lineBuffer[icol]-minValue)/(maxValue-minValue)*nbin>=nbin){
+          //   //test
+          //   std::cout << "..." << lineBuffer[icol] << std::endl;
+	  //   ++output[nbin-1];
+          // }
 	  else
 	    ++output[static_cast<int>(static_cast<double>(lineBuffer[icol]-minValue)/(maxValue-minValue)*nbin)];
 	}
@@ -316,11 +310,11 @@ int main(int argc, char *argv[])
     }
     else{
       int minCol,minRow;
-      if(min_opt[0]){
+      if(min_opt.size()){
         assert(band_opt[0]<imgReader.nrOfBand());
         std::cout << "--min " << imgReader.getMin(minCol, minRow,band_opt[0]);
       }
-      if(max_opt[0]){
+      if(max_opt.size()){
         assert(band_opt[0]<imgReader.nrOfBand());
         assert(band_opt[0]<imgReader.nrOfBand());
         std::cout << "--max " << imgReader.getMax(minCol, minRow,band_opt[0]);
@@ -383,22 +377,21 @@ int main(int argc, char *argv[])
         std::vector<float> rowBuffer;//buffer will be resized in readdata
         for(int iy=0;iy<y_opt.size();++iy){
 	  double theRow=y_opt[iy];
-          for(int ix=0;ix<x_opt.size();++ix){
-	    double theCol=x_opt[iy];
-	    if(geo_opt[0])
-	      imgReader.geo2image(x_opt[ix],y_opt[iy],theCol,theRow);
+	  int ncol=(x_opt.size())? x_opt.size() : imgReader.nrOfCol();
+          for(int ix=0;ix<ncol;++ix){
+	    double theCol=ix;
+	    if(x_opt.size()){
+	      if(geo_opt[0])
+		imgReader.geo2image(x_opt[ix],y_opt[iy],theCol,theRow);
+	      else
+		theCol=x_opt[ix];
+	    }
             assert(theRow>=0);
             assert(theRow<imgReader.nrOfRow());
             imgReader.readData(rowBuffer,GDT_Float32, static_cast<int>(theRow), theBand);
-            if(x_opt[ix]>=0){
-              assert(theCol<rowBuffer.size());
-              std::cout << rowBuffer[static_cast<int>(theCol)] << " ";
-            }
-            else{
-              for(int i=0;i<rowBuffer.size();++i)
-                std::cout << rowBuffer[i] << " ";
-            }
-          }
+	    assert(theCol<rowBuffer.size());
+	    std::cout << rowBuffer[static_cast<int>(theCol)] << " ";
+	  }
           std::cout << std::endl;
         }
       }
@@ -421,7 +414,8 @@ int main(int argc, char *argv[])
     else
       std::cout << "no intersect" << std::endl;
   }
-  //    std::cout << "bounding box mosaic (ULX ULY LRX LRY): " << minULX << " " << maxULY << " " << maxLRX << " " << minLRY << std::endl;
-  if(!read_opt[0])
+  if(!input_opt.size())
+    std::cerr << "No input file provided (use option -i). Use pkinfo --help for help information" << std::endl;
+  else if(!read_opt[0]&&!hist_opt[0])
     std::cout << std::endl;
 }
