@@ -23,102 +23,83 @@ along with pktools.  If not, see <http://www.gnu.org/licenses/>.
 #include "imageclasses/ImgWriterGdal.h"
 #include "imageclasses/ImgReaderOgr.h"
 #include "fileclasses/FileReaderLas.h"
-#include "algorithms/Histogram.h"
+#include "algorithms/StatFactory.h"
 #include "algorithms/Filter2d.h"
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 int main(int argc,char **argv) {
-  std::string versionString="version ";
-  versionString+=VERSION;
-  versionString+=", Copyright (C) 2008-2012 Pieter Kempeneers.\n\
-   This program comes with ABSOLUTELY NO WARRANTY; for details type use option -h.\n\
-   This is free software, and you are welcome to redistribute it\n\
-   under certain conditions; use option --license for details.";
-  Optionpk<bool> version_opt("\0","version",versionString,false);
-  Optionpk<bool> license_opt("lic","license","show license information",false);
-  Optionpk<bool> help_opt("h","help","shows this help info",false);
   Optionpk<string> input_opt("i", "input", "Input las file", "");
   // Optionpk<string> mask_opt("m", "mask", "mask image file", "");
-  // Optionpk<short> invalid_opt("t", "invalid", "Mask value(s) where image is invalid. Use multiple values for a single mask. Default value is 0", 0);
-  Optionpk<short> flag_opt("f", "flag", "Flag value(s) to put in image if not valid. Use as many flags as invalid options (Default is 0)", 0);
-  Optionpk<string> attribute_opt("n", "name", "names of the attribute to select: [intensity|return|nreturn|z]", "z");
-  Optionpk<bool> disc_opt("circ", "circular", "circular disc kernel for dilation and erosion (default is false)", false);
-  Optionpk<double> maxSlope_opt("s", "maxSlope", "Maximum slope used for morphological filtering (default is 0)", 0.0);
+  // Optionpk<short> invalid_opt("t", "invalid", "Mask value(s) where image is invalid. Use multiple values for a single mask.", 0);
+  Optionpk<short> flag_opt("f", "flag", "Flag value(s) to put in image if not valid. Use as many flags as invalid options", 0);
+  Optionpk<string> attribute_opt("n", "name", "names of the attribute to select: intensity, return, nreturn, z", "z");
+  Optionpk<bool> disc_opt("circ", "circular", "circular disc kernel for dilation and erosion", false);
+  Optionpk<double> maxSlope_opt("s", "maxSlope", "Maximum slope used for morphological filtering", 0.0);
   Optionpk<double> hThreshold_opt("ht", "maxHeight", "initial and maximum height threshold for progressive morphological filtering (e.g., -ht 0.2 -ht 2.5)", 0.2);
-  Optionpk<short> maxIter_opt("\0", "maxIter", "Maximum number of iterations in post filter (default is 100)", 100.0);
-  Optionpk<short> nbin_opt("nb", "nbin", "Number of percentile bins for calculating profile (=number of output bands) (default is 10)", 10.0);
-  Optionpk<unsigned short> returns_opt("r", "returns", "number(s) of returns to include (use -r -1 for last return only). Default is 0 (include all returns)", 0);
-  Optionpk<string> composite_opt("c", "composite", "composite for multiple points in cell (min, max, median, mean, sum, first, last, profile). Default is last (overwrite cells with latest point", "last");
-  Optionpk<string> filter_opt("fir", "filter", "filter las points (single,multiple,all). Default is all", "all");
-  Optionpk<string> postFilter_opt("pf", "pfilter", "filter las points (etew_min,promorph (progressive morphological filter),bunting (adapted promorph),open,close,none) . Default is none", "none");
-  Optionpk<short> dimx_opt("\0", "dimX", "Dimension X of postFilter (default is 3)", 3);
-  Optionpk<short> dimy_opt("\0", "dimY", "Dimension Y of postFilter (default is 3)", 3);
+  Optionpk<short> maxIter_opt("\0", "maxIter", "Maximum number of iterations in post filter", 100.0);
+  Optionpk<short> nbin_opt("nb", "nbin", "Number of percentile bins for calculating profile (=number of output bands)", 10.0);
+  Optionpk<unsigned short> returns_opt("r", "returns", "number(s) of returns to include");
+  Optionpk<unsigned short> classes_opt("c", "classes", "classes to keep: 0 (created, never classified), 1 (unclassified), 2 (ground), 3 (low vegetation), 4 (medium vegetation), 5 (high vegetation), 6 (building), 7 (low point, noise), 8 (model key-point), 9 (water), 10 (reserved), 11 (reserved), 12 (overlap)");
+  Optionpk<string> composite_opt("comp", "comp", "composite for multiple points in cell (min, max, median, mean, sum, first, last, profile, number (point density)). Last: overwrite cells with latest point", "last");
+  Optionpk<string> filter_opt("fir", "filter", "filter las points (last,single,multiple,all).", "all");
+  Optionpk<string> postFilter_opt("pf", "pfilter", "post processing filter (etew_min,promorph (progressive morphological filter),bunting (adapted promorph),open,close,none).", "none");
+  Optionpk<short> dimx_opt("\0", "dimX", "Dimension X of postFilter", 3);
+  Optionpk<short> dimy_opt("\0", "dimY", "Dimension Y of postFilter", 3);
   Optionpk<string> output_opt("o", "output", "Output image file", "");
-  Optionpk<string> projection_opt("p", "projection", "projection in EPSG code, e.g., EPSG:3035 (Default is no projection)", "");
-  Optionpk<double> ulx_opt("\0", "ulx", "Upper left x value bounding box (in geocoordinates if georef is true). Default is 0: read from input file", 0.0);
-  Optionpk<double> uly_opt("\0", "uly", "Upper left y value bounding box (in geocoordinates if georef is true). Default is 0: read from input file", 0.0);
-  Optionpk<double> lrx_opt("\0", "lrx", "Lower right x value bounding box (in geocoordinates if georef is true). Default is 0: read from input file", 0.0);
-  Optionpk<double> lry_opt("\0", "lry", "Lower right y value bounding box (in geocoordinates if georef is true). Default is 0: read from input file", 0.0);
+  Optionpk<string> projection_opt("p", "projection", "projection in EPSG code, e.g., EPSG:3035", "");
+  Optionpk<double> ulx_opt("\0", "ulx", "Upper left x value bounding box (in geocoordinates if georef is true). 0 is read from input file", 0.0);
+  Optionpk<double> uly_opt("\0", "uly", "Upper left y value bounding box (in geocoordinates if georef is true). 0 is read from input file", 0.0);
+  Optionpk<double> lrx_opt("\0", "lrx", "Lower right x value bounding box (in geocoordinates if georef is true). 0 is read from input file", 0.0);
+  Optionpk<double> lry_opt("\0", "lry", "Lower right y value bounding box (in geocoordinates if georef is true). 0 is read from input file", 0.0);
   Optionpk<string> otype_opt("ot", "otype", "Data type for output image ({Byte/Int16/UInt16/UInt32/Int32/Float32/Float64/CInt16/CInt32/CFloat32/CFloat64}). Empty string: inherit type from input image", "Byte");
   Optionpk<string> oformat_opt("of", "oformat", "Output image format (see also gdal_translate). Empty string: inherit from input image", "GTiff");
   Optionpk<string> option_opt("co", "co", "options: NAME=VALUE [-co COMPRESS=LZW] [-co INTERLEAVE=BAND]", "INTERLEAVE=BAND");
-  Optionpk<double> dx_opt("dx", "dx", "Output resolution in x (in meter) (default is 0.0: keep original resolution)", 0.0);
-  Optionpk<double> dy_opt("dy", "dy", "Output resolution in y (in meter) (default is 0.0: keep original resolution)", 0.0);
+  Optionpk<double> dx_opt("dx", "dx", "Output resolution in x (in meter)", 1.0);
+  Optionpk<double> dy_opt("dy", "dy", "Output resolution in y (in meter)", 1.0);
   Optionpk<string> colorTable_opt("ct", "ct", "color table (file with 5 columns: id R G B ALFA (0: transparent, 255: solid)", "");
-  Optionpk<short> verbose_opt("v", "verbose", "verbose (default is 0)", 0);
+  Optionpk<short> verbose_opt("v", "verbose", "verbose mode", 0);
 
-  version_opt.retrieveOption(argc,argv);
-  license_opt.retrieveOption(argc,argv);
-  help_opt.retrieveOption(argc,argv);
-
-  if(version_opt[0]){
-    cout << version_opt.getHelp() << endl;
+  bool doProcess;//stop process when program was invoked with help option (-h --help)
+  try{
+    doProcess=input_opt.retrieveOption(argc,argv);
+    // mask_opt.retrieveOption(argc,argv);
+    // invalid_opt.retrieveOption(argc,argv);
+    flag_opt.retrieveOption(argc,argv);
+    attribute_opt.retrieveOption(argc,argv);
+    disc_opt.retrieveOption(argc,argv);
+    maxSlope_opt.retrieveOption(argc,argv);
+    hThreshold_opt.retrieveOption(argc,argv);
+    maxIter_opt.retrieveOption(argc,argv);
+    nbin_opt.retrieveOption(argc,argv);
+    returns_opt.retrieveOption(argc,argv);
+    classes_opt.retrieveOption(argc,argv);
+    composite_opt.retrieveOption(argc,argv);
+    filter_opt.retrieveOption(argc,argv);
+    postFilter_opt.retrieveOption(argc,argv);
+    dimx_opt.retrieveOption(argc,argv);
+    dimy_opt.retrieveOption(argc,argv);
+    output_opt.retrieveOption(argc,argv);
+    projection_opt.retrieveOption(argc,argv);
+    ulx_opt.retrieveOption(argc,argv);
+    uly_opt.retrieveOption(argc,argv);
+    lrx_opt.retrieveOption(argc,argv);
+    lry_opt.retrieveOption(argc,argv);
+    otype_opt.retrieveOption(argc,argv);
+    oformat_opt.retrieveOption(argc,argv);
+    option_opt.retrieveOption(argc,argv);
+    dx_opt.retrieveOption(argc,argv);
+    dy_opt.retrieveOption(argc,argv);
+    colorTable_opt.retrieveOption(argc,argv);
+    verbose_opt.retrieveOption(argc,argv);
+  }
+  catch(string predefinedString){
+    std::cout << predefinedString << std::endl;
     exit(0);
   }
-  if(license_opt[0]){
-    cout << Optionpk<bool>::getGPLv3License() << endl;
-    exit(0);
+  if(!doProcess){
+    std::cout << "short option -h shows basic options only, use long option --help to show all options" << std::endl;
+    exit(0);//help was invoked, stop processing
   }
 
-  input_opt.retrieveOption(argc,argv);
-  // mask_opt.retrieveOption(argc,argv);
-  // invalid_opt.retrieveOption(argc,argv);
-  flag_opt.retrieveOption(argc,argv);
-  attribute_opt.retrieveOption(argc,argv);
-  disc_opt.retrieveOption(argc,argv);
-  maxSlope_opt.retrieveOption(argc,argv);
-  hThreshold_opt.retrieveOption(argc,argv);
-  maxIter_opt.retrieveOption(argc,argv);
-  nbin_opt.retrieveOption(argc,argv);
-  returns_opt.retrieveOption(argc,argv);
-  composite_opt.retrieveOption(argc,argv);
-  filter_opt.retrieveOption(argc,argv);
-  postFilter_opt.retrieveOption(argc,argv);
-  dimx_opt.retrieveOption(argc,argv);
-  dimy_opt.retrieveOption(argc,argv);
-  output_opt.retrieveOption(argc,argv);
-  projection_opt.retrieveOption(argc,argv);
-  ulx_opt.retrieveOption(argc,argv);
-  uly_opt.retrieveOption(argc,argv);
-  lrx_opt.retrieveOption(argc,argv);
-  lry_opt.retrieveOption(argc,argv);
-  otype_opt.retrieveOption(argc,argv);
-  oformat_opt.retrieveOption(argc,argv);
-  option_opt.retrieveOption(argc,argv);
-  dx_opt.retrieveOption(argc,argv);
-  dy_opt.retrieveOption(argc,argv);
-  colorTable_opt.retrieveOption(argc,argv);
-  verbose_opt.retrieveOption(argc,argv);
-
-  if(help_opt[0]){
-    cout << version_opt.getHelp() << endl;
-    exit(0);
-  }
-
-
   GDALAllRegister();
 
   double dfComplete=0.0;
@@ -128,6 +109,7 @@ int main(int argc,char **argv) {
   double progress=0;
 
   Vector2d<vector<float> > inputData;//row,col,point
+
    
   ImgReaderGdal maskReader;
   ImgWriterGdal outputWriter;
@@ -235,6 +217,12 @@ int main(int argc,char **argv) {
 
   inputData.clear();
   inputData.resize(nrow,ncol);
+  Vector2d<float> outputData(nrow,ncol);
+  for(int irow=0;irow<nrow;++irow)
+    for(int icol=0;icol<ncol;++icol)
+      outputData[irow][icol]=0;
+
+
   std::cout << "Reading " << input_opt.size() << " las files" << std::endl;
   pfnProgress(progress,pszMessage,pProgressArg);
   for(int iinput=0;iinput<input_opt.size();++iinput){
@@ -249,8 +237,10 @@ int main(int argc,char **argv) {
     //set bounding filter
     // lasReader.addBoundsFilter(minULX,maxULY,maxLRX,minLRY);
     //set returns filter
-    if(returns_opt[0])
+    if(returns_opt.size())
       lasReader.addReturnsFilter(returns_opt);
+    if(classes_opt.size())
+      lasReader.addClassFilter(classes_opt);
     lasReader.setFilters();
 
     if(attribute_opt[0]!="z"){
@@ -288,15 +278,27 @@ int main(int argc,char **argv) {
         continue;
       if((filter_opt[0]=="multiple")&&(thePoint.GetNumberOfReturns()<2))
         continue;
+      if(filter_opt[0]=="last"){
+        if(thePoint.GetReturnNumber()!=thePoint.GetNumberOfReturns())
+          continue;
+      }
       double dcol,drow;
       outputWriter.geo2image(thePoint.GetX(),thePoint.GetY(),dcol,drow);
       int icol=static_cast<int>(dcol);
       int irow=static_cast<int>(drow);
+      // //test
+      // irow+=1;
+      if(irow<0||irow>=nrow)
+	continue;
+      if(icol<0||icol>=ncol)
+	continue;
       assert(irow>=0);
       assert(irow<nrow);
       assert(icol>=0);
       assert(icol<ncol);
-      if(attribute_opt[0]=="z")
+      if(composite_opt[0]=="number")
+        outputData[irow][icol]+=1;
+      else if(attribute_opt[0]=="z")
         inputData[irow][icol].push_back(thePoint.GetZ());
       else if(attribute_opt[0]=="intensity")
         inputData[irow][icol].push_back(thePoint.GetIntensity());
@@ -320,32 +322,33 @@ int main(int argc,char **argv) {
   std::cout << "processing LiDAR points" << std::endl;
   progress=0;
   pfnProgress(progress,pszMessage,pProgressArg);
-  Histogram hist;
-  //fill in inputData in outputData
-  Vector2d<float> outputData(nrow,ncol);
+  statfactory::StatFactory stat;
+  //fill inputData in outputData
   if(composite_opt[0]=="profile"){
     assert(postFilter_opt[0]=="none");
     // for(int iband=0;iband<nband;++iband)
       // outputProfile[iband].resize(nrow,ncol);
   }
   for(int irow=0;irow<nrow;++irow){
+    if(composite_opt[0]=="number")
+      continue;//outputData already set
     Vector2d<float> outputProfile(nband,ncol);
     for(int icol=0;icol<ncol;++icol){
       std::vector<float> profile;
       if(!inputData[irow][icol].size())
         outputData[irow][icol]=(static_cast<float>((flag_opt[0])));
       else{
-        Histogram hist;
+        statfactory::StatFactory stat;
         if(composite_opt[0]=="min")
-          outputData[irow][icol]=hist.min(inputData[irow][icol]);
+          outputData[irow][icol]=stat.min(inputData[irow][icol]);
         else if(composite_opt[0]=="max")
-          outputData[irow][icol]=hist.max(inputData[irow][icol]);
+          outputData[irow][icol]=stat.max(inputData[irow][icol]);
         else if(composite_opt[0]=="median")
-          outputData[irow][icol]=hist.median(inputData[irow][icol]);
+          outputData[irow][icol]=stat.median(inputData[irow][icol]);
         else if(composite_opt[0]=="mean")
-          outputData[irow][icol]=hist.mean(inputData[irow][icol]);
+          outputData[irow][icol]=stat.mean(inputData[irow][icol]);
         else if(composite_opt[0]=="sum")
-          outputData[irow][icol]=hist.sum(inputData[irow][icol]);
+          outputData[irow][icol]=stat.sum(inputData[irow][icol]);
         else if(composite_opt[0]=="first")
           outputData[irow][icol]=inputData[irow][icol][0];
         else if(composite_opt[0]=="last")
@@ -358,11 +361,11 @@ int main(int argc,char **argv) {
           }
           float min=0;
           float max=0;
-          hist.minmax(inputData[irow][icol],inputData[irow][icol].begin(),inputData[irow][icol].end(),min,max);
+          stat.minmax(inputData[irow][icol],inputData[irow][icol].begin(),inputData[irow][icol].end(),min,max);
           if(verbose_opt[0])
             std::cout << "min,max: " << min << "," << max << std::endl;
           if(max>min){
-            hist.percentiles(inputData[irow][icol],inputData[irow][icol].begin(),inputData[irow][icol].end(),profile,nband,min,max);
+            stat.percentiles(inputData[irow][icol],inputData[irow][icol].begin(),inputData[irow][icol].end(),profile,nband,min,max);
             assert(profile.size()==nband);
             for(int iband=0;iband<nband;++iband)
               outputProfile[iband][icol]=profile[iband];
@@ -383,7 +386,7 @@ int main(int argc,char **argv) {
         // assert(outputProfile[iband].size()==outputWriter.nrOfRow());
         assert(outputProfile[iband].size()==outputWriter.nrOfCol());
         try{
-          outputWriter.writeData(outputProfile[iband],theType,irow,iband);
+          outputWriter.writeData(outputProfile[iband],GDT_Float32,irow,iband);
         }
         catch(std::string errorString){
           cout << errorString << endl;
@@ -408,14 +411,14 @@ int main(int argc,char **argv) {
     //increase cells and thresholds until no points from the previous iteration are discarded.
     int dimx=dimx_opt[0];
     int dimy=dimy_opt[0];
-    Filter2d::Filter2d morphFilter;
+    filter2d::Filter2d morphFilter;
     morphFilter.setNoValue(0);
     Vector2d<float> currentOutput=outputData;
     int iteration=1;
     while(nchange&&iteration<maxIter_opt[0]){
       double hThreshold=maxSlope_opt[0]*dimx;
       Vector2d<float> newOutput;
-      nchange=morphFilter.morphology(currentOutput,newOutput,Filter2d::ERODE,dimx,dimy,disc_opt[0],hThreshold);
+      nchange=morphFilter.morphology(currentOutput,newOutput,"erode",dimx,dimy,disc_opt[0],hThreshold);
       currentOutput=newOutput;
       dimx+=2;//change from theory: originally double cellCize
       dimy+=2;//change from theory: originally double cellCize
@@ -433,7 +436,7 @@ int main(int argc,char **argv) {
     //increase cells and thresholds until no points from the previous iteration are discarded.
     int dimx=dimx_opt[0];
     int dimy=dimy_opt[0];
-    Filter2d::Filter2d theFilter;
+    filter2d::Filter2d theFilter;
     theFilter.setNoValue(0);
     Vector2d<float> currentOutput=outputData;
     double hThreshold=hThreshold_opt[0];
@@ -442,21 +445,19 @@ int main(int argc,char **argv) {
       std::cout << "iteration " << iteration << " with window size " << dimx << " and dh_max: " << hThreshold << std::endl;
       Vector2d<float> newOutput;
       try{
-        theFilter.morphology(outputData,currentOutput,Filter2d::ERODE,dimx,dimy,disc_opt[0],maxSlope_opt[0]);
-        theFilter.morphology(currentOutput,outputData,Filter2d::DILATE,dimx,dimy,disc_opt[0],maxSlope_opt[0]);
-      //   if(postFilter_opt[0]=="bunting"){//todo: implement doit in Filter2d on Vector2d
-      //     theFilter.doit(outputData,currentOutput,Filter2d::MEDIAN,dimx,dimy,1,disc_opt[0]);
-      // filter2d.doit(input,output,Filter2d::MEDIAN,dimX_opt[0],dimY_opt[0],down_opt[0],disc_opt[0]);
-
-      //     outputData=currentOutput;
-      //   }
+        theFilter.morphology(outputData,currentOutput,"erode",dimx,dimy,disc_opt[0],maxSlope_opt[0]);
+        theFilter.morphology(currentOutput,outputData,"dilate",dimx,dimy,disc_opt[0],maxSlope_opt[0]);
+        if(postFilter_opt[0]=="bunting"){//todo: implement doit in Filter2d on Vector2d
+          theFilter.doit(outputData,currentOutput,"median",dimx,dimy,1,disc_opt[0]);
+          outputData=currentOutput;
+        }
       }
       catch(std::string errorString){
         cout << errorString << endl;
         exit(1);
       }
-      int newdimx=2*(dimx-1)+1;
-      int newdimy=2*(dimy-1)+1;//from PE&RS vol 71 pp313-324
+      int newdimx=(dimx==1)? 3: 2*(dimx-1)+1;
+      int newdimy=(dimx==1)? 3: 2*(dimy-1)+1;//from PE&RS vol 71 pp313-324
       hThreshold=hThreshold_opt[0]+maxSlope_opt[0]*(newdimx-dimx)*dx_opt[0];
       dimx=newdimx;
       dimy=newdimy;
@@ -469,12 +470,12 @@ int main(int argc,char **argv) {
   else if(postFilter_opt[0]=="open"){
     if(composite_opt[0]!="min")
       std::cout << "Warning: composite option is not set to min!" << std::endl;
-    Filter2d::Filter2d morphFilter;
+    filter2d::Filter2d morphFilter;
     morphFilter.setNoValue(0);
     Vector2d<float> filterInput=outputData;
     try{
-      morphFilter.morphology(outputData,filterInput,Filter2d::ERODE,dimx_opt[0],dimy_opt[0],disc_opt[0],maxSlope_opt[0]);
-      morphFilter.morphology(filterInput,outputData,Filter2d::DILATE,dimx_opt[0],dimy_opt[0],disc_opt[0],maxSlope_opt[0]);
+      morphFilter.morphology(outputData,filterInput,"erode",dimx_opt[0],dimy_opt[0],disc_opt[0],maxSlope_opt[0]);
+      morphFilter.morphology(filterInput,outputData,"dilate",dimx_opt[0],dimy_opt[0],disc_opt[0],maxSlope_opt[0]);
     }
     catch(std::string errorString){
       cout << errorString << endl;
@@ -484,12 +485,12 @@ int main(int argc,char **argv) {
   else if(postFilter_opt[0]=="close"){
     if(composite_opt[0]!="max")
       std::cout << "Warning: composite option is not set to max!" << std::endl;
-    Filter2d::Filter2d morphFilter;
+    filter2d::Filter2d morphFilter;
     morphFilter.setNoValue(0);
     Vector2d<float> filterInput=outputData;
     try{
-      morphFilter.morphology(outputData,filterInput,Filter2d::DILATE,dimx_opt[0],dimy_opt[0],disc_opt[0],maxSlope_opt[0]);
-      morphFilter.morphology(filterInput,outputData,Filter2d::ERODE,dimx_opt[0],dimy_opt[0],disc_opt[0],maxSlope_opt[0]);
+      morphFilter.morphology(outputData,filterInput,"dilate",dimx_opt[0],dimy_opt[0],disc_opt[0],maxSlope_opt[0]);
+      morphFilter.morphology(filterInput,outputData,"erode",dimx_opt[0],dimy_opt[0],disc_opt[0],maxSlope_opt[0]);
     }
     catch(std::string errorString){
       cout << errorString << endl;
@@ -505,7 +506,7 @@ int main(int argc,char **argv) {
       try{
         assert(outputData.size()==outputWriter.nrOfRow());
         assert(outputData[0].size()==outputWriter.nrOfCol());
-        outputWriter.writeData(outputData[irow],theType,irow,0);
+        outputWriter.writeData(outputData[irow],GDT_Float32,irow,0);
       }
       catch(std::string errorString){
         cout << errorString << endl;
