@@ -22,6 +22,11 @@ along with pktools.  If not, see <http://www.gnu.org/licenses/>.
 #include <time.h>
 #include "ogr_spatialref.h"
 #include "ImgWriterGdal.h"
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 //---------------------------------------------------------------------------
 ImgWriterGdal::ImgWriterGdal(void)
   : m_gds(NULL), m_isGeoRef(false), m_ncol(0), m_nrow(0), m_nband(0)
@@ -106,6 +111,7 @@ void ImgWriterGdal::setCodec(const ImgReaderGdal& imgSrc){
   }
   char **papszMetadata;
   papszMetadata = poDriver->GetMetadata();
+  //todo: try and catch if CREATE is not supported (as in PNG)
   assert( CSLFetchBoolean( papszMetadata, GDAL_DCAP_CREATE, FALSE ));
   char **papszOptions=NULL;
   for(vector<string>::const_iterator optionIt=m_options.begin();optionIt!=m_options.end();++optionIt)
@@ -126,7 +132,10 @@ void ImgWriterGdal::setCodec(const ImgReaderGdal& imgSrc){
   }
   m_gds->SetMetadata(imgSrc.getMetadata() ); 
   m_gds->SetMetadataItem( "TIFFTAG_DOCUMENTNAME", m_filename.c_str());
-  m_gds->SetMetadataItem( "TIFFTAG_SOFTWARE", "pktools v2.0 by Pieter Kempeneers");
+  std::string versionString="pktools ";
+  versionString+=VERSION;
+  versionString+=" by Pieter Kempeneers";
+  m_gds->SetMetadataItem( "TIFFTAG_SOFTWARE", versionString.c_str());
   time_t rawtime;
   time ( &rawtime );
 
@@ -181,6 +190,7 @@ void ImgWriterGdal::setCodec(const string& imageType)
   }
   char **papszMetadata;
   papszMetadata = poDriver->GetMetadata();
+  //todo: try and catch if CREATE is not supported (as in PNG)
   assert( CSLFetchBoolean( papszMetadata, GDAL_DCAP_CREATE, FALSE ));
   char **papszOptions=NULL;
   for(vector<string>::const_iterator optionIt=m_options.begin();optionIt!=m_options.end();++optionIt)
@@ -196,7 +206,10 @@ void ImgWriterGdal::setCodec(const string& imageType)
   // m_gds->SetMetadataItem( "INTERLEAVE", m_interleave.c_str(), "IMAGE_STRUCTURE" );
   // m_gds->SetMetadataItem( "COMPRESSION", m_compression.c_str(), "IMAGE_STRUCTURE" );
   m_gds->SetMetadataItem( "TIFFTAG_DOCUMENTNAME", m_filename.c_str());
-  m_gds->SetMetadataItem( "TIFFTAG_SOFTWARE", "pktools v2.0 by Pieter Kempeneers");
+  std::string versionString="pktools ";
+  versionString+=VERSION;
+  versionString+=" by Pieter Kempeneers";
+  m_gds->SetMetadataItem( "TIFFTAG_SOFTWARE", versionString.c_str());
   time_t rawtime;
   time ( &rawtime );
 
